import os, fileinput, string, math, time, sys, shutil, glob, subprocess
from xml.dom import minidom


def HoloVizio2MpegConvertor_LinearLayout(xmlFileName, mpegParamFileName):
# Converting XML Linear HoloVizio 3D Camera system file 
# to MPEG camera parameter file format:
# MPEG format:
#         camera_name
#         Intrinsic parameter matrix K 3x3
#         0.0  "Origin for corresponding image" 
#         0.0
#         extrinsic parameter matrix P = [R T] 3x4

	parametersFromXmlFile = ['count','PathLength', 'CameraDistance', 'CameraFovX', \
		'CameraAspect', 'CameraWidth', 'CameraHeight']
	parameterValuesFromXmlFile = [None]* len(parametersFromXmlFile)
# Extract Parameters from XML HoloVizio file:
	xmlFile = minidom.parse(xmlFileName)
	for p in range(0, len(parametersFromXmlFile)):
		if (p == 0):
			CameraLayout = xmlFile.getElementsByTagName("CameraLayout")
			parameterValue = int(CameraLayout[0].getAttribute("count"))
		else:
			#print xmlFile.getElementsByTagName(parametersFromXmlFile[p])
			parameterValue = float(xmlFile.getElementsByTagName(parametersFromXmlFile[p])[0].firstChild.data)
		parameterValuesFromXmlFile[p] = float(parameterValue)
	numOfCamera = int(parameterValuesFromXmlFile[0])
	width = parameterValuesFromXmlFile[5]
	height = parameterValuesFromXmlFile[6]
	focalLength = 0.5*width/(math.tan(math.radians(parameterValuesFromXmlFile[3]/2)));
	cameraInterval = parameterValuesFromXmlFile[1]/numOfCamera;
# Write MPEG Camera Parameter file:
	mpegParamFile = open(mpegParamFileName, 'w')
	for i in range(0, numOfCamera):
		mpegParamFile.write ('\nparam_ptm%02d \n' % i)
		K = [[focalLength,0,width/2], \
			[0,focalLength,height/2],[0,0,1]];
		P = [[1, 0,0,(cameraInterval*(i-float(numOfCamera-1)/2))],[0, 1, 0, 0],[0, 0, 1, 0]]
		for r in range(0, len(K)):
			for c in range(0, len(K)):
				mpegParamFile.write(str(K[r][c]))
				mpegParamFile.write(' ')
			mpegParamFile.write('\n')
		mpegParamFile.write('0.0\n')
		mpegParamFile.write('0.0\n')
		for r in range(0, len(P[:])):
			for c in range(0, len(P[:][1])):
				mpegParamFile.write(str(P[r][c]))
				mpegParamFile.write(' ')
			mpegParamFile.write('\n')
	mpegParamFile.close()
	return (numOfCamera, width, height)


def HoloVizio2MpegConvertor_ArcLayout(xmlFileName, mpegParamFileName):
# Converting XML Arc HoloVizio 3D Camera system file 
# to MPEG camera parameter file format:
# MPEG format:
#         camera_name
#         Intrinsic parameter matrix K 3x3
#         0.0  "Origin for corresponding image" 
#         0.0
#         extrinsic parameter matrix P = [R T] 3x4

	parametersFromXmlFile = ['count','Angle', 'Radius', 'CameraFovX', \
		'CameraWidth', 'CameraHeight']
	parameterValuesFromXmlFile = [None]* len(parametersFromXmlFile)
# Extract Parameters from XML HoloVizio file:
	xmlFile = minidom.parse(xmlFileName)
	for p in range(0, len(parametersFromXmlFile)):
		if (p == 0):
			CameraLayout = xmlFile.getElementsByTagName("CameraLayout")
			parameterValue = int(CameraLayout[0].getAttribute("count"))
		else:
			parameterValue = float(xmlFile.getElementsByTagName(parametersFromXmlFile[p])[0].firstChild.data)
		parameterValuesFromXmlFile[p] = float(parameterValue)
	numOfCamera = int(parameterValuesFromXmlFile[0])
	angle = parameterValuesFromXmlFile[1]
	radius = parameterValuesFromXmlFile[2]
	cameraFovX = parameterValuesFromXmlFile[3]
	width = parameterValuesFromXmlFile[4]
	height = parameterValuesFromXmlFile[5]
	focalLength = 0.5*width/(math.tan(math.radians(cameraFovX/2)));
	firstCameraLocationInDegree = -0.5*angle
	angularResolution = angle/(numOfCamera-1)
	beta = firstCameraLocationInDegree 
# Write MPEG Camera Parameter file:
	mpegParamFile = open(mpegParamFileName, 'w')
	for i in range(0, numOfCamera):
		mpegParamFile.write ('\nparam_ptm%02d \n' % i)
		K = [[focalLength,0,width/2], \
			[0,focalLength,height/2],[0,0,1]]
		xt = radius*math.sin(math.radians(beta))
		yt = 0;
		zt = radius*math.cos(math.radians(beta))
		P = [[math.cos(math.radians(beta)), 0, math.sin(math.radians(beta)), xt]\
			,[0, 1, 0, yt]\
			,[-math.sin(math.radians(beta)), 0, math.cos(math.radians(beta)), zt]]
		beta  += angularResolution
		for r in range(0, len(K)):
			for c in range(0, len(K)):
				mpegParamFile.write(str(K[r][c]))
				mpegParamFile.write(' ')
			mpegParamFile.write('\n')
		mpegParamFile.write('0.0\n')
		mpegParamFile.write('0.0\n')
		for r in range(0, len(P[:])):
			for c in range(0, len(P[:][1])):
				mpegParamFile.write(str(P[r][c]))
				mpegParamFile.write(' ')
			mpegParamFile.write('\n')
	mpegParamFile.close()
	return (numOfCamera, width, height)


HoloVizio2MpegConvertor_LinearLayout('BigBuckBunny_Flowers_Linear.xml', 'BigBuckBunny_Flowers_Linear.txt')
HoloVizio2MpegConvertor_ArcLayout('BigBuckBunny_Flowers_Arc.xml', 'BigBuckBunny_Flowers_Arc.txt')
HoloVizio2MpegConvertor_LinearLayout('BigBuckBunny_Butterfly_Linear.xml', 'BigBuckBunny_Butterfly_Linear.txt')
HoloVizio2MpegConvertor_ArcLayout('BigBuckBunny_Butterfly_Arc.xml', 'BigBuckBunny_Butterfly_Arc.txt')
HoloVizio2MpegConvertor_LinearLayout('BigBuckBunny_Rabbit_Linear.xml', 'BigBuckBunny_Rabbit_Linear.txt')
HoloVizio2MpegConvertor_ArcLayout('BigBuckBunny_Rabbit_Arc.xml', 'BigBuckBunny_Rabbit_Arc.txt')