# import struct
# Reads a Y-only yuv video file, and converts it to yuv420

# ----------- Read the input file ------------
# all data elements are 1 byte(s) long
NbytesPerElement = 1
# size of one frame in number of pixels
NpixelsPerFrame = 1600 * 1200
# first, last and no. of frames to read
StartFrame = 0         # starts at 0 for the first frame
NumberOfFrames = 1267  # NumberOfFrames>500 heavily slows down the processing (couple of minutes)
StopFrame = StartFrame+NumberOfFrames-1

# ----------- File definition ----------------
FilenameBase = 'images/depth-7-8bit-zn300-zf3000'
f_in = open(FilenameBase + '.yuv', 'rb')
f_out = open(FilenameBase +'_Y420_'+ str(StartFrame) +'_'+ str(StopFrame) +'.yuv', 'wb')

# UV as a chunck of successive values of 128
UV = bytearray([128]*(NpixelsPerFrame>>1)*NbytesPerElement)

# start
print('Processing started') 

# go to the first frame
f_in.seek((StartFrame)*NpixelsPerFrame*NbytesPerElement)

# Copy frame per frame, adding UV components per frame
for i in range(NumberOfFrames):
    # copy one frame to the output
    OneFrame = f_in.read(NpixelsPerFrame*NbytesPerElement)
    f_out.write(OneFrame)
    # add U and V component per frame
    f_out.write(UV)

# close all files
f_in.close()
f_out.close()

# done
print('Done!')

